/**
	@file 	pGina.c
	@brief 	pGina support functions.
*/
/*
pGina - A modular GINA Interface for Windows
Copyright (C) 2002 Nathan Yocom, Michael Wright

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: yocomnw@plu.edu
Web: http://pgina.cs.plu.edu
Snail Mail:
  Nathan Yocom @ CSCE
  Pacific Lutheran University
  Tacoma WA, 98447
  Phone: 253-535-7408
*/

#include <windows.h>
#include <winwlx.h>
#include <stdio.h>
#include <Winreg.h>
#include "PGina.h"
#include "resource.h"

/**
  @brief Called when the pGina DLL is first attached by WinLogon (or other program)
  
  Called when the pGina DLL is first attached by WinLogon (or other program)
  we save the hInstance for later use in mapping to other functions etc.   
  
  @author 	Nathan Yocom
  @param 	hInstance	The instance of this particular DLL loading
  @param	dwReason	The reason for being attached/detached
  @param	lpReserved	Reserved by Microsoft (c)   
  @retval 	boolean
	Unless there is a major problem, return true
*/
BOOL WINAPI DllMain(HINSTANCE hInstance,DWORD dwReason,LPVOID lpReserved)
{
	switch(dwReason)					  // Decide why we have entered 
	{
	case DLL_PROCESS_ATTACH:			  // If we entered and are being attached to 
		pGinahInstance = hInstance;		  // Save our parents hInstance
		pGinaLoadInfo(&settingsInfo);	  // Load configuration from reg
		debugOut(TEXT("pGina.dll loaded."));	  // Write out the info
		debugOut(VERSION);				  // and our version
		hDll = NULL;
		// Fall through and return true
	case DLL_PROCESS_DETACH:

	default: 
		return(TRUE);
	}
}
/*******/

/**
  @brief 	Configure pGina settings

	The pGinaConfigure() method is called by the external configuration application.  
	This method is responsible for providing a GUI interface and functionality for 
	the configuration of the pGina DLL and its settings.  Settings include all members
	of the pGinaInfo structure except the username and password.
  @author	Nathan Yocom
  @retval 	boolean
	In the event of any errors, this method will provide all necessary warnings and 
	information to the user prior to returning a false value, otherwise a value of true
	will be returned.
  
  @par Example Usage
	<pre>
	if(pGinaConfigure()) {
		// Configuration was successful, continue will processing
	}
	else {	   
		// There was an error in configuration, the user should have been given details
		exit(-1);
	}</pre> 

	@see ConfigBoxProc()
*/
EXTERNAL_API BOOL pGinaConfigure(void)
{
	return DialogBox(pGinahInstance,MAKEINTRESOURCE(IDD_CONFIGURE),NULL,(DLGPROC)ConfigBoxProc);    
}

/**
	@brief 	Load a pGina plugin and map functions within it
	@author Nathan Yocom
	
	The pGinaPluginLoad() method is called to map all the necessary methods from the 
	plugin defined within the passed pGinaInfo structure. Those methods will 
	be mapped as defined by the pGina Plugin Specifications Document at http://pgina.cs.plu.edu

	@param	settingsInfo	A pointer to a pGinaInfo structure that contains at least
	the pGinaInfo::pathPlugin element.
	  
	@see pGinaInfo::pathPlugin
  
	@retval	boolean
	In the event of any errors, this method will provide all necessary warnings and 
	information to the user prior to returning a false value, otherwise a value of true
	will be returned.
  
	@par Example Usage
	<pre>
	pGinaInfo settingsInfo;
	settingsInfo.pathPlugin = "C:\\PGINA\\PLUGINS\\PLUGIN.DLL"
	if(!pGinaLoadPlugin(&settingsInfo)) {
		// There was an error in loading the plugin and the user has been notified
		exit(-1);		
	}</pre>
	
*/
BOOL pGinaPluginLoad(pGinaInfo *settingsInfo)
{
	if(hDll != NULL)
	{
		pluginUserLogin = NULL;
		pluginChangeUserPassword = NULL;
		pluginAboutPlugin = NULL;
		pluginChangePluginSettings = NULL;
		pluginLoginHook = NULL;
		pluginLogoutHook = NULL;

		FreeLibrary(hDll);
	}

	// Make sure its a DLL and set the handle
	if(!(hDll = LoadLibrary(settingsInfo->pathPlugin)))
	{
		// Notify the user if there were issues
		MessageBox(NULL,TEXT("This does not appear to be a valid plugin."),TEXT("Error"),MB_OK);
		return false;
	}
	else
	{
		// Map the methods
		pluginUserLogin = (DLLUSERLOGIN) GetProcAddress(hDll,"UserLogin");
		pluginChangeUserPassword = (DLLCHANGEUSERPASSWORD) GetProcAddress(hDll,"ChangeUserPassword");
		pluginAboutPlugin = (DLLABOUTPLUGIN) GetProcAddress(hDll,"AboutPlugin");
		pluginChangePluginSettings = (DLLCHANGEPLUGINSETTINGS) GetProcAddress(hDll,"ChangePluginSettings");
		pluginLoginHook = (DLLLOGINHOOK) GetProcAddress(hDll,"LoginHook");
		pluginLogoutHook = (DLLLOGOUTHOOK) GetProcAddress(hDll,"LogoutHook");

		// If any of them were NOT loaded
		if(!pluginUserLogin || !pluginChangeUserPassword || !pluginAboutPlugin || !pluginChangePluginSettings || !pluginLoginHook || !pluginLogoutHook)
		{
			// Tell the user something was missing/wrong
			MessageBox(NULL,TEXT("This does not appear to be a valid or complete pGina plugin."),TEXT("Error Loading"),MB_OK);
			return false;
		}
		else
		{
			// If we get here things are cool...
			return true;
		}
	}
	// if we get this far something is DEFINETLY wrong!
	return false; 
}

/**
	@brief	Map network resources to local logical drives
	@author Nathan Yocom
	
	The pGinaMapDrives() method is called to parse and map all drives as specified 
	in the pGinaInfo structure.

	@param	settingsInfo	A pointer to a pGinaInfo structure

	@retval	boolean
	In the event of any errors, this method will provide all necessary warnings and 
	information to the user prior to returning a false value, otherwise a value of true
	will be returned.
  
	@par Example Usage
	<pre>
	pGinaInfo settingsInfo;
	settingsInfo.mapPaths = "X:\\\\host.name\\path\\%username%"
	if(!pGinaMapDrives(&settingsInfo)) {
		// An error occurred mapping local devices to remote devices
		// The user has been notified - time to decide whether we continue or not...
	}</pre>

	@bug High latency networks seem to result in non-mapped connections.
	Sleep is a temporary fix.
	
	@see explode_vars()
	@see pGinaInfo::mapPaths
*/
BOOL pGinaMapDrives(pGinaInfo *settingsInfo)
{
	TCHAR *current;		// Current entry
	TCHAR *list;			// The full list
	TCHAR drive[3];		// The drive to map
	TCHAR *path;			// the path to map to *drive
	LONG lResult;

	NETRESOURCE NetSettings;  // The structure for mapping info

	NetSettings.dwType = RESOURCETYPE_DISK;
	NetSettings.lpProvider = NULL;
	NetSettings.dwDisplayType   = RESOURCEDISPLAYTYPE_SHARE;
	NetSettings.dwScope         = RESOURCE_GLOBALNET;
	NetSettings.dwUsage         = RESOURCEUSAGE_CONNECTABLE;
	NetSettings.lpComment       = NULL;

	// Copy the main list (we don't want to screw with that!)
	list = _tcsdup(settingsInfo->mapPaths);

	// Get the first item...
	current = _tcstok(list,TEXT(";"));
	// Until there are no more items
	while(current != NULL)
	{
		// Explode any variables...
		explode_vars(&current, settingsInfo);               

		// Current now has "DRIVE:\\Share\"

		path = _tcsdup(current+2);
		_tcsncpy(drive,current,2);
		drive[2] = '\0';

		NetSettings.lpLocalName = drive;
		NetSettings.lpRemoteName = path;

		debugOut(TEXT("Sleeping for 5 seconds"));
		Sleep(5000);	// Takes milliseconds as argument

		debugOut(TEXT("Trying to map:"));
		debugOut(drive);
		debugOut(path);

		// All variables are there now - DO THE MAP
		if(!ImpersonateLoggedOnUser(hUser))
		{
			debugOut(TEXT("Impersonation failed."));
		}
		lResult = WNetAddConnection2(&NetSettings,(LPCTSTR)settingsInfo->Password,(LPCTSTR) settingsInfo->Username,CONNECT_INTERACTIVE);


		if(lResult != NO_ERROR)
		{

			debugOut(TEXT("That didnt work. trying to get info from user"));
			switch(lResult)
			{
			case ERROR_ACCESS_DENIED:
				debugOut(TEXT("ERROR_ACCESS_DENIED"));
				break;
			case ERROR_ALREADY_ASSIGNED:
				debugOut(TEXT("ERROR_ALREADY_ASSIGNED"));
				break;
			case ERROR_BAD_DEV_TYPE:
				debugOut(TEXT("ERROR_BAD_DEV_TYPE"));
				break;
			case ERROR_BAD_DEVICE:
				debugOut(TEXT("ERROR_BAD_DEVICE"));
				break;
			case ERROR_BAD_NET_NAME:
				debugOut(TEXT("ERROR_BAD_NET_NAME"));
				break;
			case ERROR_BAD_PROFILE:
				debugOut(TEXT("ERROR_BAD_PROFILE"));
				break;
			case ERROR_BAD_PROVIDER:
				debugOut(TEXT("ERROR_BAD_PROVIDER"));
				break;
			case ERROR_BUSY:
				debugOut(TEXT("ERROR_BUSY"));
				break;
			case ERROR_CANCELLED:
				debugOut(TEXT("ERROR_CANCELLED"));
				break;
			case ERROR_CANNOT_OPEN_PROFILE:
				debugOut(TEXT("ERROR_CANNOT_OPEN_PROFILE"));
				break;
			case ERROR_EXTENDED_ERROR:
				debugOut(TEXT("ERROR_EXTENDED_ERROR"));
				break;
			case ERROR_INVALID_PASSWORD:
				debugOut(TEXT("ERROR_INVALID_PASSWORD"));
				break;
			case ERROR_NO_NET_OR_BAD_PATH:
				debugOut(TEXT("ERROR_NO_NET_OR_BAD_PATH"));
				break;
			case ERROR_NO_NETWORK:
				debugOut(TEXT("ERROR_NO_NETWORK"));
				break;
			default:
				debugOut(TEXT("Unknown Map Error"));
			}
			lResult = WNetAddConnection2(&NetSettings,NULL,NULL,CONNECT_PROMPT | CONNECT_INTERACTIVE);
		}

		if(lResult != NO_ERROR)
		{
			// There was an error
			_tcscat(path,TEXT("*"));
			switch(lResult)
			{
			case ERROR_ACCESS_DENIED:
				MessageBox(NULL,TEXT("ERROR_ACCESS_DENIED"),path,MB_OK);
				break;
			case ERROR_ALREADY_ASSIGNED:
				MessageBox(NULL,TEXT("ERROR_ALREADY_ASSIGNED"),path,MB_OK);
				break;
			case ERROR_BAD_DEV_TYPE:
				MessageBox(NULL,TEXT("ERROR_BAD_DEV_TYPE"),path,MB_OK);
				break;
			case ERROR_BAD_DEVICE:
				MessageBox(NULL,TEXT("ERROR_BAD_DEVICE"),path,MB_OK);
				break;
			case ERROR_BAD_NET_NAME:
				MessageBox(NULL,TEXT("ERROR_BAD_NET_NAME"),path,MB_OK);
				break;
			case ERROR_BAD_PROFILE:
				MessageBox(NULL,TEXT("ERROR_BAD_PROFILE"),path,MB_OK);
				break;
			case ERROR_BAD_PROVIDER:
				MessageBox(NULL,TEXT("ERROR_BAD_PROVIDER"),path,MB_OK);
				break;
			case ERROR_BUSY:
				MessageBox(NULL,TEXT("ERROR_BUSY"),path,MB_OK);
				break;
			case ERROR_CANCELLED:
				MessageBox(NULL,TEXT("ERROR_CANCELLED"),path,MB_OK);
				break;
			case ERROR_CANNOT_OPEN_PROFILE:
				MessageBox(NULL,TEXT("ERROR_CANNOT_OPEN_PROFILE"),path,MB_OK);
				break;
			case ERROR_EXTENDED_ERROR:
				MessageBox(NULL,TEXT("ERROR_EXTENDED_ERROR"),path,MB_OK);
				break;
			case ERROR_INVALID_PASSWORD:
				MessageBox(NULL,TEXT("ERROR_INVALID_PASSWORD"),path,MB_OK);
				break;
			case ERROR_NO_NET_OR_BAD_PATH:
				MessageBox(NULL,TEXT("ERROR_NO_NET_OR_BAD_PATH"),path,MB_OK);
				break;
			case ERROR_NO_NETWORK:
				MessageBox(NULL,TEXT("ERROR_NO_NETWORK"),path,MB_OK);
				break;
			default:
				MessageBox(NULL,TEXT("Unknown Map Error"),path,MB_OK);
			}

			return false;
		}
		// ----------------------------------------

		// Get next token
		current = _tcstok(NULL ,TEXT(";"));
	}

	// If we get this far, then all went well
	return true;
}

/**
	@brief	Remove network drives
	@author Nathan Yocom
	
	The pGinaDelDrives() method is called to parse and remove connections
	to all drives as specified in the pGinaInfo structure.

	@param	settingsInfo	A pointer to a pGinaInfo structure

	@retval	boolean
	In the event of any errors, this method will provide all necessary warnings and 
	information to the user prior to returning a false value, otherwise a value of true
	will be returned.
  
	@see pGinaInfo::mapPaths
*/
BOOL pGinaDelDrives(pGinaInfo *settingsInfo)
{
	TCHAR *current;		// Current entry
	TCHAR *list;			// The full list
	TCHAR drive[3];		// The drive to map
	TCHAR *path;			// the path to map to *drive
	DWORD lResult;
	BOOL somethingFailed = false;

	// Copy the main list (we don't want to screw with that!)
	list = _tcsdup(settingsInfo->mapPaths);

	// Get the first item...
	current = _tcstok(list,TEXT(";"));
	// Until there are no more items
	while(current != NULL)
	{
		path = _tcsdup(current+2);
		_tcsncpy(drive,current,2);
		drive[2] = '\0';

		// lpName is now in drive[]

		debugOut(TEXT("Trying to remove connection to %s"),drive);

		// All variables are there now - DO THE MAP
/*		if(!ImpersonateLoggedOnUser(hUser))
		{
			debugOut(TEXT("Impersonation failed."));
		} */

		lResult = WNetCancelConnection2((LPCTSTR)drive,0, TRUE);

		if(lResult != NO_ERROR)
		{

			debugOut(TEXT("That didnt work."));
			somethingFailed = true;
		}
		// ----------------------------------------

		// Get next token
		current = _tcstok(NULL ,TEXT(";"));
	}

	// If we get this far, then all went well
	if(!somethingFailed)
		return true;
	else
		return false;
}

/**
	@brief 	Replace varname with variable values
	@author Nathan Yocom
	
	Replaces \%varname% with appropriate variable values
	
	@param	current	A pointer to a TCHAR array that contains the string
	to explode values in.
	@param Info	A pointer to a pGinaInfo structure to use for variable
	substitution.
	
	@par Example Usage
	<pre>
	TCHAR example[] = "\\Example\\Path\\%username%";
	pGinaInfo settingsInfo;
	settingsInfo.Username = "myUsername";
	explode_vars(&example,&settingsInfo);
	// At this point example[] = "\\Example\\Path\\myUsername"; </pre>

  @note
	Only substitutes for \%username%, we do not feel other variables need to be
	included.  However, if this is not the case, please contact the developers.
*/
void explode_vars(TCHAR **current,pGinaInfo *Info)
{
	TCHAR *variable_start;
	TCHAR *variable_end;
	TCHAR *exploded_path;

	// Check for and explode variables, only %username% is known right now...
	if((variable_start = _tcsstr(*current,TEXT("%"))) != NULL)
	{
		// Set markers at the start and end of the %username
		variable_end = _tcsstr(variable_start,TEXT("%"));

		// Setup a new variable for the path
		exploded_path = (TCHAR *) malloc(_tcslen(*current) + _tcslen(Info->Username) + _tcslen(variable_end) +1);

		// Put a NULL where the % is to end our first string
		*variable_start = '\0';

		// Copy the first portion (before the %)
		_tcscpy(exploded_path,*current);
		// Then the username
		_tcscat(exploded_path,Info->Username);
		// Then the last portion
		_tcscat(exploded_path,variable_end);
		*current = _tcsdup(exploded_path);
	}
}

/**
	@brief	Save settings information to the registry
	@author	Michael Wright

	The pGinaSaveInfo() method is called when information within the pGinaInfo structure 
	passed needs to be saved to the registry.  The information will be stored in the Windows 
	registry under the key \\HKLM\SOFTWARE\pGina\ and will be readable by all users and 
	writable only to local administrator accounts.

	@param	settingsInfo	A pointer to a pGinaInfo structure.

	@retval	boolean
	In the event of any errors, this method will provide all necessary warnings and 
	information to the user prior to returning a false value, otherwise a value of true
	will be returned.
  
	@par Example Usage
	<pre>
	pGinaInfo settingsInfo;
	settingsInfo.mapPaths = "X:\\\\host.name\\path\\%username%"
	...
	...
	(etc)
	if(!pGinaSaveInfo(&settingsInfo)) {
		// An error occurred saving the passed settings
		// The user has been notified - time to decide whether we continue or not...
	}</pre>
	@note	Key's are created by Administrator, and can only be read (not changed) by 
	"normal" users.
*/
BOOL pGinaSaveInfo(pGinaInfo *settingsInfo)
{
	//gets the lengths of the strings to be copied to the registry
	/*DWORD dwMSGina = (_tcslen(settingsInfo->pathMSGina)+1);
	DWORD dwMapped = (_tcslen(settingsInfo->mapPaths)+1);
	DWORD dwProfile = (_tcslen(settingsInfo->pathProfile)+1);
	DWORD dwPlugin = (_tcslen(settingsInfo->pathPlugin)+1);*/
	DWORD dwMSGina = MAX_PATH;
	DWORD dwMapped = MAX_PATH;
	DWORD dwProfile = MAX_PATH;
	DWORD dwPlugin = MAX_PATH;
	DWORD dwTitle = MAX_PATH;
	DWORD dwMotd = MAX_PATH;
	DWORD dwLogo = MAX_PATH;
	DWORD one = 1;
	LONG lResult;
	HKEY hKey;
	DWORD T = 1;
	DWORD F = 0;


	lResult = RegCreateKeyEx(HKEY_LOCAL_MACHINE,
							 _T("SOFTWARE\\pGina"),
							 0,
							 NULL,
							 REG_OPTION_NON_VOLATILE,
							 KEY_ALL_ACCESS,
							 NULL,	 // This field can be used to set Security..?
							 &hKey,
							 NULL);	 // This value can be used to determine if the key existed or not..
									 //  but we dont care as long as it opens.. i guess.

	if(lResult == ERROR_SUCCESS)
	{
		
		RegSetValueEx(hKey,_T("title"),0,REG_SZ,(CONST BYTE *) windowTitle, dwTitle);
		RegSetValueEx(hKey,_T("motd"),0,REG_SZ,(CONST BYTE *) motdText, dwMotd);
		RegSetValueEx(hKey,_T("logo"),0,REG_SZ,(CONST BYTE *) customLogo, dwLogo);
		
		
		lResult = RegSetValueEx(hKey,_T("pathMSGina"),0,REG_SZ,
								(CONST BYTE *) settingsInfo->pathMSGina,
								dwMSGina);
		if(lResult == ERROR_SUCCESS)
		{
			lResult = RegSetValueEx(hKey,_T("mapPaths"),0,REG_SZ,
									(CONST BYTE *) settingsInfo->mapPaths,
									dwMapped);
			if(lResult == ERROR_SUCCESS)
			{
				lResult = RegSetValueEx(hKey,_T("pathProfile"),0,REG_SZ,
										(CONST BYTE *) settingsInfo->pathProfile,
										dwProfile);
				if(lResult == ERROR_SUCCESS)
				{
					lResult = RegSetValueEx(hKey,_T("pathPlugin"),0,REG_SZ,
											(CONST BYTE *) settingsInfo->pathPlugin,
											dwPlugin);

					if(lResult == ERROR_SUCCESS)
					{
						if(!debugOutput)
							lResult = RegSetValueEx(hKey,_T("debugOutput"),0,REG_DWORD,(LPBYTE)&F,sizeof(DWORD));
						else
							lResult	= RegSetValueEx(hKey,_T("debugOutput"),0,REG_DWORD,(LPBYTE)&T,sizeof(DWORD));
					}

					if(lResult == ERROR_SUCCESS)
					{
						if(!keepProfiles)
							lResult = RegSetValueEx(hKey,_T("keepProfiles"),0,REG_DWORD,&F,sizeof(DWORD));
						else
							lResult	= RegSetValueEx(hKey,_T("keepProfiles"),0,REG_DWORD,&T,sizeof(DWORD));
					}
					if(lResult == ERROR_SUCCESS)
					{
						if(!settingsInfo->disabled)
							lResult = RegSetValueEx(hKey,_T("passThru"),0,REG_DWORD,&F,sizeof(DWORD));
						else
							lResult	= RegSetValueEx(hKey,_T("passThru"),0,REG_DWORD,&T,sizeof(DWORD));
					}
					if(lResult == ERROR_SUCCESS)
					{
						if(!allowLocking)
							lResult = RegSetValueEx(hKey,_T("allowLocking"),0,REG_DWORD,&F,sizeof(DWORD));
						else
							lResult	= RegSetValueEx(hKey,_T("allowLocking"),0,REG_DWORD,&T,sizeof(DWORD));
					}
					if(lResult == ERROR_SUCCESS)
					{
						if(!mapAdmin)
							lResult = RegSetValueEx(hKey,_T("mapAdmin"),0,REG_DWORD,&F,sizeof(DWORD));
						else
							lResult	= RegSetValueEx(hKey,_T("mapAdmin"),0,REG_DWORD,&T,sizeof(DWORD));
					}
					if(lResult == ERROR_SUCCESS)
					{
						if(!buttonAction)
							lResult = RegSetValueEx(hKey,_T("buttonAction"),0,REG_DWORD,&F,sizeof(DWORD));
						else
							lResult	= RegSetValueEx(hKey,_T("buttonAction"),0,REG_DWORD,&T,sizeof(DWORD));
					}
					if(lResult == ERROR_SUCCESS)
					{
						if(!forceLocal)
							lResult = RegSetValueEx(hKey,_T("forceLogin"),0,REG_DWORD,&F,sizeof(DWORD));
						else
							lResult	= RegSetValueEx(hKey,_T("forceLogin"),0,REG_DWORD,&T,sizeof(DWORD));
					}
					if(lResult == ERROR_SUCCESS)
					{
						if(!hidePlugin)
							lResult = RegSetValueEx(hKey,_T("hidePlugin"),0,REG_DWORD,&F,sizeof(DWORD));
						else
							lResult	= RegSetValueEx(hKey,_T("hidePlugin"),0,REG_DWORD,&T,sizeof(DWORD));

						if(!hideMotd)
							lResult = RegSetValueEx(hKey,_T("hideMotd"),0,REG_DWORD,&F,sizeof(DWORD));
						else
							lResult	= RegSetValueEx(hKey,_T("hideMotd"),0,REG_DWORD,&T,sizeof(DWORD));
					}
				}
			}
		}
		
		lResult = RegFlushKey(hKey);
		RegCloseKey(hKey);
		return true;
	}
	else
	{
		return false;
	}
}

/**
	@brief	Load settings information from the registry
	@author	Michael Wright
	
	The pGinaLoadInfo() method is called when information within the pGinaInfo structure 
	passed needs to be loaded from the registry.  The information will be stored in the Windows 
	registry under the key \\HKLM\Software\pGina\ and will be readable by all users and 
	writable only to local administrator accounts.

	@param	settingsInfo	A pointer to a pGinaInfo structure to fill with the registry values

	@retval boolean
	In the event of any errors, this method will provide all necessary warnings and 
	information to the user prior to returning a false value, otherwise a value of true
	will be returned.
  
	@par Example Usage
	<pre>
	pGinaInfo settingsInfo;
	settingsInfo.mapPaths = "X:\\\\host.name\\path\\%username%"
	...
	...
	(etc)
	if(!pGinaLoadInfo(&settingsInfo)) {
		// An error occurred getting the settings
		// The user has been notified - time to decide whether we continue or not...
	}</pre>

	@Note	Key's are created by Administrator, and can only be read (not changed) by 
	"normal" users.
*/
BOOL pGinaLoadInfo(pGinaInfo *settingsInfo)
{

	DWORD dwMSGina = 2048;
	DWORD dwMapped = 2048;
	DWORD dwProfile = 2048;
	DWORD dwPlugin = 2048;
	DWORD five = 5;
	DWORD Value = 0;
	DWORD size = sizeof(DWORD);

	LONG lResult;
	HKEY hKey;

	TCHAR pathMSGina[2048];
	TCHAR mapPaths[2048];
	TCHAR pathProfile[2048];
	TCHAR pathPlugin[2048];

	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina"),
						   0,
						   KEY_QUERY_VALUE,
						   &hKey);

	if(lResult == ERROR_SUCCESS)
	{
		lResult = RegQueryValueEx(hKey,_T("pathMSGina"),
								  NULL,
								  NULL,
								  (LPBYTE) pathMSGina,
								  &dwMSGina);

		if(lResult == ERROR_SUCCESS)
		{
			settingsInfo->pathMSGina = _tcsdup(pathMSGina);

			lResult = RegQueryValueEx(hKey,_T("mapPaths"),
									  NULL,
									  NULL,
									  (LPBYTE) mapPaths,
									  &dwMapped);

			if(lResult == ERROR_SUCCESS)
			{
				settingsInfo->mapPaths = _tcsdup(mapPaths);

				lResult = RegQueryValueEx(hKey,_T("pathPlugin"),
										  NULL,
										  NULL,
										  (LPBYTE) pathPlugin,
										  &dwPlugin);

				if(lResult == ERROR_SUCCESS)
				{
					settingsInfo->pathPlugin = _tcsdup(pathPlugin);

					lResult = RegQueryValueEx(hKey,_T("pathProfile"),
											  NULL,
											  NULL,
											  (LPBYTE) pathProfile,
											  &dwProfile);

					if(lResult == ERROR_SUCCESS)
					{
						settingsInfo->pathProfile = _tcsdup(pathProfile);

						lResult = RegQueryValueEx(hKey,_T("debugOutput"),
												  NULL,
												  NULL,
												  (LPBYTE) &Value,
												  &size);

						if(lResult == ERROR_SUCCESS)
						{
							if(Value == 1)
								debugOutput = true;
							else
								debugOutput	= false;

							lResult = RegQueryValueEx(hKey,_T("keepProfiles"),
													  NULL,
													  NULL,
													  (LPBYTE) &Value,
													  &size);

							if(lResult == ERROR_SUCCESS)
							{
								if(Value == 1)
									keepProfiles = true;
								else
									keepProfiles = false;

								lResult = RegQueryValueEx(hKey,_T("passThru"),
														  NULL,
														  NULL,
														  (LPBYTE) &Value,
														  &size);

								if(lResult == ERROR_SUCCESS)
								{
									if(Value == 1)
										settingsInfo->disabled = true;
									else
										settingsInfo->disabled = false;

									lResult = RegQueryValueEx(hKey,_T("allowLocking"),
															  NULL,
															  NULL,
															  (LPBYTE) &Value,
															  &size);

									if(lResult == ERROR_SUCCESS)
									{
										if(Value == 1)
											allowLocking = true;
										else
											allowLocking = false;

										lResult = RegQueryValueEx(hKey,_T("mapAdmin"),
																  NULL,
																  NULL,
																  (LPBYTE) &Value,
																  &size);

										if(lResult == ERROR_SUCCESS)
										{
											if(Value == 1)
												mapAdmin = true;
											else
												mapAdmin = false;

											lResult = RegQueryValueEx(hKey,_T("buttonAction"),
																	  NULL,
																	  NULL,
																	  (LPBYTE) &Value,
																	  &size);

											if(lResult == ERROR_SUCCESS)
											{
												if(Value == 1)
													buttonAction = true;
												else
													buttonAction = false;

												if(lResult == ERROR_SUCCESS)
												{
													lResult = RegQueryValueEx(hKey,_T("forceLogin"),
																			  NULL,
																			  NULL,
																			  (LPBYTE) &Value,
																			  &size);

													if(lResult == ERROR_SUCCESS)
													{
														if(Value == 1)
															forceLocal = true;
														else
															forceLocal = false;

														//// START NON-REQUIRED KEYS 
														lResult = RegQueryValueEx(hKey,_T("hideMotd"),
																			  NULL,
																			  NULL,
																			  (LPBYTE) &Value,
																			  &size);

														if(lResult == ERROR_SUCCESS)
														{
															if(Value == 1)
																hideMotd = true;
															else
																hideMotd = false;
														}
														else
															hideMotd = false;

														lResult = RegQueryValueEx(hKey,_T("hidePlugin"),
																			  NULL,
																			  NULL,
																			  (LPBYTE) &Value,
																			  &size);

														if(lResult == ERROR_SUCCESS)
														{
															if(Value == 1)
																hidePlugin = true;
															else
																hidePlugin = false;
														}
														else
															hidePlugin = false;
														/// END NON-REQUIRED KEYS
													}
												}
											}
										}
									}
								}
							}
						}
					}

				}
			}
		}
		RegCloseKey(hKey);

		return true;
	}

	return false;


}

/**
	@brief	Load user profile from disk, to correct location
	@author Nathan Yocom
	
	The pGinaLoadProfile() method is called load the users profile from the path 
	specified in the pGinaInfo structure.  In the event that this profile does not 
	exist, a default should be created.

	@note Not yet implemented
	
*/
BOOL pGinaLoadProfile(pGinaInfo *settingsInfo)
{
	TCHAR *path;
	TCHAR path_local[MAX_PATH];
	int ret;

	debugOut(TEXT("in LoadProfile()!"));
	
    if(settingsInfo->pathProfile != NULL)
	{
		// settingsInfo.pathProfile = Z:\%username%
		path = _tcsdup(settingsInfo->pathProfile);
		explode_vars(&path,settingsInfo);

		_stprintf(path_local,TEXT("C:\\Documents and Settings\\%s"),settingsInfo->Username);

		ret = CopyFolder((LPCTSTR)path,(LPCTSTR)path_local, TRUE);

		debugOut(TEXT("CopyFolder returned: %d"),ret);

		return true;
	}
	else
	{
		debugOut(TEXT("pathProfile == NULL"));
		return false;
	}
	
	// If we get here we are in trouble
	return true;
}

/**
	@brief	Saves user profile to disk, to correct location
	@author Nathan Yocom
	
	The pGinaSaveProfile() method is called to save the users profile from the default
	to that specified in the pGinaInfo structure.  

	@note Not yet implemented
*/
BOOL pGinaSaveProfile(pGinaInfo *settingsInfo)
{
	TCHAR *path;
	TCHAR path_local[MAX_PATH];
	TCHAR path_profile[MAX_PATH];
	int ret;

	debugOut(TEXT("in SaveProfile()!"));
   		
	if(settingsInfo->pathProfile != NULL)
	{
		path = _tcsdup(settingsInfo->pathProfile);
		explode_vars(&path,settingsInfo);

		_stprintf(path_local,TEXT("C:\\Documents and Settings\\%s"),settingsInfo->Username);
		
		ret = CopyFolder((LPCTSTR)path_local,(LPCTSTR)path, TRUE);

		debugOut(TEXT("CopyFolder returned: %d"),ret);

		DeleteFolder(path_local);  // Remove the C:\Documents and Settings\ folder		

		return true;
	}
	else
	{
		return false;
	}
}


/**
	@brief	Writes passed info to debug file
	@author	Nathan Yocom
	
	@param	string	The string (TCHAR []) to be output to the file
	
	This function outputs the passed string to a debug text file stored in C:\
	
	@note	If debugOutput is not defined this function does nothing
	@note	Updated with 1.0 to actually use printf style strings (ala "Something %s",variable)
	@see	debugOutput
*/
void debugOut(TCHAR *string,...)
{
	TCHAR filename[256];
	int number = 0;
	va_list argList;
	time_t ltime;

	time(&ltime);

	if(debugOutput)
	{
		if(debugFile == NULL)
		{
			number = time(NULL);
			_stprintf(filename,TEXT("c:\\pgina_debug%d.txt"),number);
			debugFile = _tfopen(filename,TEXT("w"));
		}

		va_start(argList,string);
		_ftprintf(debugFile,TEXT("@%.*s: "),24,ctime(&ltime));
		_vftprintf(debugFile,string,argList);
		_ftprintf(debugFile,TEXT("\n"));
		fflush(debugFile);

	}
}
/**
	@brief	Sets the credentials for AutoLogin
	@author	Jeremy C. Sparks (Willamette University), jcsparks@willamette.edu
	@note	Code re-used with permission from WuGina project, by Jeremy C. Sparks.
	@version	1.7.3
	
	Sets the username and password portions of the autologin registry.
	
	@param 	AutoUser	The username to autologin with
	@param	AutoPassword	The password to use
	@param	UserFullName	The fullname of the user
*/
void SetAutoCredentials( PTCHAR AutoUser, PTCHAR AutoPassword, PTCHAR UserFullName)
{
	HKEY    hKey;
	LONG    lResult;
	PTCHAR  pszSystemName;

	pszSystemName = GlobalAlloc( GPTR, MAX_PATH );
	if( ! pszSystemName )
		return;

	GetEnvironmentVariable( _T("COMPUTERNAME"), pszSystemName, MAX_PATH );

	lResult = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
							_T("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon"),
							0, KEY_WRITE, &hKey );

	if(lResult == ERROR_SUCCESS)
	{
		lResult = RegSetValueEx( hKey, _T("DefaultUserName"), 0, REG_SZ,
								 (CONST BYTE*)AutoUser,
								 _tcslen( AutoUser ) * sizeof(_TCHAR)
							   );
		if(lResult == ERROR_SUCCESS)
		{
			lResult = RegSetValueEx( hKey, _T("DefaultPassword"), 0, REG_SZ,
									 (CONST BYTE*)AutoPassword,
									 _tcslen( AutoPassword ) * sizeof(_TCHAR)
								   );
/* nls
			if(lResult == ERROR_SUCCESS)
			{
				lResult = RegSetValueEx( hKey, _T("DefaultDomainName"), 0, REG_SZ,
										 (CONST BYTE*)pszSystemName,
										 _tcslen( pszSystemName ) * sizeof(_TCHAR) // this machine, no domain
									   );
				if(lResult == ERROR_SUCCESS)
				{
					lResult= RegFlushKey( hKey );
				}
				else
					debugOut(TEXT("Registry error in writing auto-login info!"));
			}
*/
		}

		RegCloseKey( hKey );
	}
	if( pszSystemName )	GlobalFree( pszSystemName );

}

/**
	@brief	Sets or removes the AutoAdmin bit
	@author	Jeremy C. Sparks (Willamette University), jcsparks@willamette.edu
	@note	Code re-used with permission from WuGina project, by Jeremy C. Sparks.
	@version	1.7.3
	
	Sets or removes the autologin registry bits.
	
	@param 	value	1 to set AutoAdmin true (use SetAutoCredentials() to set the username/password)
	0 to remove AutoAdmin bits and remove AutoCredentials (if they exist)
*/
void SetAutoAdmin( int value )
{ // Poke autoadmin stuff into registry
	HKEY    hKey;
	LONG    lResult;

	lResult = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
							_T("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon"),
							0, KEY_WRITE, &hKey );

	if(lResult == ERROR_SUCCESS)
	{
		lResult = RegSetValueEx( hKey, _T("AutoAdminLogon"), 0, REG_SZ,
								 value ? "1" : "0",
								 1 * sizeof(TCHAR));
		lResult = RegSetValueEx( hKey, _T("ForceAutoLogon"), 0, REG_SZ, "1", 1 * sizeof(TCHAR));

		// clear any leftover entries.
		//nls RegDeleteValue( hKey, _T("DefaultDomainName") );
		//RegDeleteValue( hKey, _T("DefaultUserName") );
		RegDeleteValue( hKey, _T("DefaultPassword") );

		if(lResult == ERROR_SUCCESS)
		{
			lResult= RegFlushKey( hKey );
		}
		else
			debugOut(TEXT("Error writing autoadmin bit to registry"));

		RegCloseKey( hKey );
	}
}

/**
	@brief	Creates a user account on the local machine
	@author	Nathan Yocom
	@note	Account creation fails if user already exists

	Creates a local user account with the username and password in the pGinaInfo structure
	settingsInfo.
	
	@todo	Implement roaming profile functionality, specify profile path here?
	@todo	Adding a user to the admin group is at this point English specific
			proper code to do this the right way is here: 
			http://63.236.73.79/cgi-bin/bbs/wt/showpost.pl?Board=vc&Number=359849&page=0&view=collapsed&mode=threaded&sb=5
			
*/
void CreateAccount(void)
{
	USER_INFO_1 ui;
	DWORD dwLevel = 1;
	DWORD dwError = 0;
	NET_API_STATUS nStatus;
	USER_INFO_1003 new_passwd;   
	USER_INFO_1007 new_comment;   
	USER_INFO_1011 new_fullname; 
	LOCALGROUP_MEMBERS_INFO_3 memberInfo;
	LPTSTR tempFullname;
	LPTSTR tempDescription;
	LPTSTR tempUsername;
	LPTSTR tempPassword;
	LPTSTR tempAdmin;

	tempUsername = _tcsdup(settingsInfo.Username);
	tempPassword = _tcsdup(settingsInfo.Password);
	tempFullname = _tcsdup(settingsInfo.userFullName);
	tempDescription = _tcsdup(settingsInfo.userDescription);
	tempAdmin = _tcsdup(TEXT("Administrators"));


	ui.usri1_name = (LPWSTR) tempUsername;
	memberInfo.lgrmi3_domainandname = (LPWSTR) tempUsername;
	ui.usri1_password = (LPWSTR) tempPassword;
	ui.usri1_flags = UF_SCRIPT | UF_PASSWD_CANT_CHANGE;
	ui.usri1_script_path = NULL;    
	ui.usri1_priv = USER_PRIV_USER;
	ui.usri1_comment = NULL;
	ui.usri1_home_dir = NULL;
	ui.usri1_password_age = NULL;

	debugOut(TEXT("All setup and converted -> Username: %s, Password: %s"),ui.usri1_name,ui.usri1_password);

	nStatus = NetUserAdd(NULL,1,(LPBYTE)&ui,NULL);

	dumpState();

	if(nStatus == NERR_Success)
	{
		debugOut(TEXT("User add worked..."));

		if(settingsInfo.isAdmin)
		{
			debugOut(TEXT("Trying to set user as an admin"));

			nStatus = NetLocalGroupAddMembers(NULL,tempAdmin,3,(LPBYTE) &memberInfo,1);

			if(nStatus == NERR_Success)
				debugOut(TEXT("Admin set worked!"));
			else
			{
				debugOut(TEXT("Admin set failed!"));
			} 
		}

		dumpState();
		if(tempFullname != NULL)
		{
			debugOut(TEXT("Setting users fullname: %s"),tempFullname);

			new_fullname.usri1011_full_name = (LPWSTR) tempFullname;
			nStatus = NetUserSetInfo(NULL,(LPWSTR)tempUsername,1011,(LPBYTE) &new_fullname,NULL);

			if(nStatus == NERR_Success)
				debugOut(TEXT("worked"));
			else
				debugOut(TEXT("failed nStatus = %d"),nStatus);  
		}
		else
			debugOut(TEXT("No Fullname for user"));

		if(tempDescription != NULL)
		{
			debugOut(TEXT("Setting users description: %s"),tempDescription);
			new_comment.usri1007_comment = (LPWSTR) tempDescription;
			nStatus = NetUserSetInfo(NULL,(LPWSTR) tempUsername,1007,(LPBYTE) &new_comment, NULL);

			if(nStatus == NERR_Success)
				debugOut(TEXT("worked"));
			else
				debugOut(TEXT("failed, nStatus = %d"),nStatus); 
		}
		else
			debugOut(TEXT("No description for user."));

		if(settingsInfo.userGroups != NULL)
		{
			debugOut(TEXT("Adding user %s to %s"),settingsInfo.Username,settingsInfo.userGroups);
			AddtoGroups();
		}
	}
	else
	{
		debugOut(TEXT("User add failed...."));
		switch(nStatus)
		{
		case NERR_InvalidComputer:
			debugOut(TEXT("NERR_InvalidComputer"));
			break;
		case NERR_NotPrimary:
			debugOut(TEXT("NERR_NotPrimary"));
			break;
		case NERR_GroupExists:
			debugOut(TEXT("NERR_GroupExists"));
			break;
		case NERR_UserExists:
			debugOut(TEXT("NERR_UserExists"));

			if(forceLocal)
			{
				debugOut(TEXT("Forcing login for %s"),settingsInfo.Username);

				new_passwd.usri1003_password = tempPassword;
				nStatus = NetUserSetInfo(NULL,(LPCWSTR) tempUsername,1003,(LPBYTE) &new_passwd,NULL);

				if(nStatus == NERR_Success)
					debugOut(TEXT("It worked... yeah! :-)"));
				else
					debugOut(TEXT("Nope.. still need to work on recovery from: nStatus = %d"),nStatus);
			}
			break;
		case NERR_PasswordTooShort:
			debugOut(TEXT("NERR_PasswordTooShort"));
			break;
		default:
			debugOut(TEXT("Crap.. who knows.."));
			debugOut(TEXT("nStatus = %d, dwError = %d"),nStatus,dwError);
			break;
		}
	} 
}

/**
	@brief	Deletes a user account on the local machine
	@author	Nathan Yocom

	Deletes a local user account with the username in the pGinaInfo structure
	settingsInfo.
	
	@todo	Implement roaming profile functionality, specify profile path here?
	@todo	Implement locating users profile directory
	@bug	Currently explicitly removes c:\documents and settings\\<username>, doesnt locate "true" path
*/

void DeleteAccount(void)
{
	NET_API_STATUS nStatus;
	LPUSER_INFO_3 uInfo = NULL; 
	TCHAR pathProfile[MAX_PATH];
	LPTSTR tempUsername;

	tempUsername = _tcsdup(settingsInfo.Username);

	nStatus = NetUserDel(NULL,tempUsername);

	// Seeing as how it seems to be impossible to determine where
	//  a users profile is stored (i tried NetUserGetInfo)... lets
	//  just delete C:\Documents and Settings\%username% for now.
	//  this needs to be FIXED!

	if(nStatus != NERR_Success)
		debugOut(TEXT("DeleteAccount Failed"));
	else
	{
		debugOut(TEXT("DeleteAccount was successfull"));

		_stprintf(pathProfile,TEXT("C:\\Documents and Settings\\%s"),tempUsername);

		debugOut(TEXT("Deleting folder..."));
		DeleteFolder(pathProfile);
		debugOut(TEXT("Done"));
	}
}

/**
	@brief	Recursivley copies a folder
	@author	Unknown - from www.codeguru.com forums
	
	Copies a folders contents recursivley
	
	@param lpszSrc Source directory
	@param lpszDes Destination directory
	@param bOverWrite BOOL indicating whether destination should be overwritten
*/
int CopyFolder(LPCTSTR lpszSrc, LPCTSTR lpszDes, BOOL bOverWrite)
{
	int ret=0;
	TCHAR name1[256];
	TCHAR szDesPath[_MAX_PATH];
	WIN32_FIND_DATA info;
	HANDLE hp;
	TCHAR *cp;

	debugOut(TEXT("CopyFolder: From %s to %s"),lpszSrc,lpszDes);
	debugOut(TEXT("CopyFolder: creating destination"));
	
	if(!CreateDirectory(lpszDes,NULL))
		if(GetLastError() != 183) //ERROR_ALREADY_EXISTS
		{
			debugOut(TEXT("CopyFolder: Somethin didnt work, getlast error says: %d"),GetLastError());
		    return ++ret;
		}

	_stprintf(name1, TEXT("%s\\*.*"),lpszSrc);
	hp = FindFirstFile(name1,&info);
    if(!hp || hp==INVALID_HANDLE_VALUE)
		return ++ret; 
	do
	{
		cp = info.cFileName;
		
		// Does this work in UNICODE?
		
		// Deal with .
		if(info.cFileName[0] == '.' && info.cFileName[1] == '\0')
			continue;
		// and ..
		if(info.cFileName[2] == '\0' && info.cFileName[0] == '.' && info.cFileName[1] == '.')
			continue;

		_stprintf(name1,TEXT("%s\\%s"),lpszSrc,info.cFileName);
		if(info.dwFileAttributes&FILE_ATTRIBUTE_READONLY)
		{
			SetFileAttributes(name1,info.dwFileAttributes&~FILE_ATTRIBUTE_READONLY);
		}
		if(info.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)
		{
			_stprintf(szDesPath,TEXT("%s\\%s"),lpszDes,info.cFileName);
			ret += CopyFolder(name1,szDesPath,bOverWrite);
		}
		else
		{
			_stprintf(szDesPath,TEXT("%s\\%s"),lpszDes,info.cFileName);
			if(!CopyFile(name1,szDesPath,!bOverWrite))
				ret++;
		}

	}
	while(FindNextFile(hp,&info));
	FindClose(hp);
	if(info.dwFileAttributes&FILE_ATTRIBUTE_READONLY)
	{
		SetFileAttributes(lpszSrc,info.dwFileAttributes&~FILE_ATTRIBUTE_READONLY);
	}
	
	RevertToSelf();
	return ret;
}

/**
	@brief	Deletes a folder recursivley
	@author	Nathan Yocom
	
	Deletes the passed folder recursivley (everything including .? files)
	
	@param directory A char[] array to the directory to be deleted
	
	@par Example Usage
	<pre>
	DeleteFolder("c:\\\\testdir")
	Results in the deletion of c:\\testdir and everything in it
	</pre>
*/
void DeleteFolder(LPCTSTR directory)
{
	WIN32_FIND_DATA FileInformation;
	HANDLE hFile;
	TCHAR fullPath[MAX_PATH];

	_tcscpy(fullPath,directory);
	_tcscat(fullPath,TEXT("\\*.*"));

	hFile = FindFirstFile(fullPath,&FileInformation);
	if(hFile != INVALID_HANDLE_VALUE)
	{
		do
		{
			if((((FileInformation.cFileName[0]) == '.') &&
				((FileInformation.cFileName[1]) != '.') &&
				/*Changed NULL to '\0' produced warning on indirection levels*/
				((FileInformation.cFileName[1]) != '\0'))
			   || ((FileInformation.cFileName[0]) != '.'))
			{
				_tcscpy(fullPath,directory);
				_tcscat(fullPath,TEXT("\\"));
				_tcscat(fullPath,FileInformation.cFileName);

				if(FileInformation.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					DeleteFolder((LPCTSTR)fullPath);    
				}
				else
				{
					if(SetFileAttributes(fullPath,FILE_ATTRIBUTE_NORMAL) == FALSE)
						return;

					if(DeleteFile(fullPath) == FALSE)
						return;

				}
			}
		} while(FindNextFile(hFile,&FileInformation) == TRUE);

		FindClose(hFile);

		if(SetFileAttributes(directory,FILE_ATTRIBUTE_NORMAL) == FALSE)
			return;

		if(RemoveDirectory(directory) == FALSE)
			return;

	}
}

/**
	@brief	Dumps any and all state info to debug log 
	@author	Nathan Yocom
*/
void dumpState(void)
{
	// First lets walk through settingsInfo

	debugOut(TEXT("---------------------- State Dump ---------------------------"));
	debugOut(TEXT("settingsInfo:"));

	if(settingsInfo.allowPassChange)
		debugOut(TEXT("\tallowPassChange = TRUE"));
	else
		debugOut(TEXT("\tallowPassChange = FALSE"));

	if(settingsInfo.authType == AUTH_LOCAL)
		debugOut(TEXT("\tauthType = AUTH_LOCAL"));
	else
		debugOut(TEXT("\tauthType = AUTH_PLUGIN"));

	if(settingsInfo.disabled)
		debugOut(TEXT("\tdisabled = TRUE"));
	else
		debugOut(TEXT("\tdisabled = FALSE"));

	if(settingsInfo.homeDir != NULL)
		debugOut(TEXT("\thomeDir = *%s*"),settingsInfo.homeDir);
	else
		debugOut(TEXT("\thomeDir = NULL"));

	if(settingsInfo.hUser == NULL)
		debugOut(TEXT("\thUser = NULL"));
	else
		debugOut(TEXT("\thUser = <Has Value>"));

	if(settingsInfo.isAdmin)
		debugOut(TEXT("\tisAdmin = TRUE"));
	else
		debugOut(TEXT("\tisAdmin = FALSE"));

	if(settingsInfo.mapPaths == NULL)
		debugOut(TEXT("\tmapPaths = NULL"));
	else
		debugOut(TEXT("\tmapPaths = *%s*"),settingsInfo.mapPaths);

	if(settingsInfo.Password == NULL)
		debugOut(TEXT("\tPassword = NULL"));
	else
		debugOut(TEXT("\tPassword = *%s*"),settingsInfo.Password);

	if(settingsInfo.pathMSGina == NULL)
		debugOut(TEXT("\tpathMSGina = NULL"));
	else
		debugOut(TEXT("\tpathMSGina = *%s*"),settingsInfo.pathMSGina);

	if(settingsInfo.pathPlugin == NULL)
		debugOut(TEXT("\tpathPlugin = NULL"));
	else
		debugOut(TEXT("\tpathPlugin = *%s*"),settingsInfo.pathPlugin);

	if(settingsInfo.pathProfile == NULL)
		debugOut(TEXT("\tpathProfile = NULL"));
	else
		debugOut(TEXT("\tpathProfile = *%s*"),settingsInfo.pathProfile);

	if(settingsInfo.userDescription == NULL)
		debugOut(TEXT("\tuserDescription = NULL"));
	else
		debugOut(TEXT("\tuserDescription = *%s*"),settingsInfo.userDescription);

	if(settingsInfo.userFullName == NULL)
		debugOut(TEXT("\tuserFullName = NULL"));
	else
		debugOut(TEXT("\tuserFullName = *%s*"),settingsInfo.userFullName);

	if(settingsInfo.userGroups == NULL)
		debugOut(TEXT("\tuserGroups = NULL"));
	else
		debugOut(TEXT("\tuserGroups = *%s*"),settingsInfo.userGroups);

	if(settingsInfo.Username == NULL)
		debugOut(TEXT("\tUsername = NULL"));
	else
		debugOut(TEXT("\tUsername = *%s*"),settingsInfo.Username);

	debugOut(TEXT(""));
	debugOut(TEXT("State Variables:"));

	if(pGinahInstance == NULL)
		debugOut(TEXT("\tpGinahInstance = NULL"));
	else
		debugOut(TEXT("\tpGinahInstance = Initialized"));

	if(keepProfiles)
		debugOut(TEXT("\tkeepProfiles = TRUE"));
	else
		debugOut(TEXT("\tkeepProfiles = FALSE"));

	if(allowLocking)
		debugOut(TEXT("\tallowLocking = TRUE"));
	else
		debugOut(TEXT("\tallowLocking = FALSE"));

	if(mapAdmin)
		debugOut(TEXT("\tmapAdmin = TRUE"));
	else
		debugOut(TEXT("\tmapAdmin = FALSE"));

	if(buttonAction)
		debugOut(TEXT("\tbuttonAction = TRUE"));
	else
		debugOut(TEXT("\tbuttonAction = FALSE"));

	if(forceLocal)
		debugOut(TEXT("\tforceLogin = TRUE"));
	else
		debugOut(TEXT("\tforceLogin = FALSE"));

	if(defaultDomain == NULL)
		debugOut(TEXT("\tdefaultDomain = NULL"));
	else
		debugOut(TEXT("\tdefaultDomain = *%s*"),defaultDomain); 

	if(debugOutput)
		debugOut(TEXT("\tdebugOutput = TRUE"));
	else
		debugOut(TEXT("\tdebugOutput = FALSE"));


	debugOut(TEXT("---------------------- End Dump -----------------------------"));

}

/** 
		@brief  Adds user to a list of groups   
		@author Nathan Yocom   
   
		Add the user to all of the groups listed in settingsInfo.userGroups.  The list   
		should be semi-colon seperated and all groups must exist.   
   
		@bug    _tcsdup's all over the place inside a loop... do we need to clean up in between?   
*/   
void AddtoGroups(void)   
{
	LOCALGROUP_MEMBERS_INFO_3 memberInfo;   
	NET_API_STATUS nStatus;   

	LPTSTR currentGroup;   
	LPTSTR Username;   

	TCHAR *fullList;   
	TCHAR *location;   

	// Setup our username for conversion   
	Username = _tcsdup(settingsInfo.Username);   

	memberInfo.lgrmi3_domainandname = Username;   

	// Setup the username for later use   

	// First copy the whole list, so we dont screw with it   
	fullList = _tcsdup(settingsInfo.userGroups);   

	// Pull of the first in the list   
	location = _tcstok(fullList,TEXT(";"));

	while(location != NULL)
	{
		currentGroup = _tcsdup(location);   

		debugOut(TEXT("Trying to add %s to group %s"),Username,currentGroup);   

		nStatus = NetLocalGroupAddMembers(NULL,currentGroup,3,(LPBYTE) &memberInfo,1);

		if(nStatus == NERR_Success)
			debugOut(TEXT("Adding %s to group %s worked."),Username,currentGroup);

		location = _tcstok(NULL,TEXT(";"));   
	}   
} 

LPTSTR pGinaGetTitle(void)
{
	TCHAR Title[2048];
	DWORD dwTitle = 2048;
	HKEY hKey;
	LONG lResult;


	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina"),
						   0,
						   KEY_QUERY_VALUE,
						   &hKey);

	if(lResult == ERROR_SUCCESS)
	{
		lResult = RegQueryValueEx(hKey,_T("title"),
								  NULL,
								  NULL,
								  (LPBYTE) Title,
								  &dwTitle);

		if(lResult == ERROR_SUCCESS)
		{
			windowTitle = _tcsdup(Title);
			return windowTitle;
		}
		else
			return TEXT("pGina Logon");
	}
	else
		return TEXT("pGina Logon");
}

LPTSTR pGinaGetMotd(void)
{
	TCHAR Title[2048];
	DWORD dwTitle = 2048;
	HKEY hKey;
	LONG lResult;


	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina"),
						   0,
						   KEY_QUERY_VALUE,
						   &hKey);

	if(lResult == ERROR_SUCCESS)
	{
		lResult = RegQueryValueEx(hKey,_T("motd"),
								  NULL,
								  NULL,
								  (LPBYTE) Title,
								  &dwTitle);

		if(lResult == ERROR_SUCCESS)
		{
			motdText = _tcsdup(Title);
			return motdText;
		}
		else
			return TEXT("");
	}
	else
		return TEXT("");
}

BOOL pGinaLoadLogo(void)
{
	TCHAR Title[2048];
	DWORD dwTitle = 2048;
	HKEY hKey;
	LONG lResult;


	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina"),
						   0,
						   KEY_QUERY_VALUE,
						   &hKey);

	if(lResult == ERROR_SUCCESS)
	{
		lResult = RegQueryValueEx(hKey,_T("logo"),
								  NULL,
								  NULL,
								  (LPBYTE) Title,
								  &dwTitle);

		if(lResult == ERROR_SUCCESS)
		{
			// Save it out for later use...
			customLogo = _tcsdup(Title);

			logoBmp = LoadImage(NULL,(LPCSTR)Title,IMAGE_BITMAP,0,0,LR_LOADFROMFILE);
			
			if(logoBmp == NULL)
			{
				return FALSE;
			}
			else
			{
				return TRUE;
			}
		}
		else
			return FALSE;
	}
	else
		return FALSE;
}

BOOL pGinaHidePlugin(void)
{
	DWORD hidePlugin = 0;
	DWORD size = 5;
	HKEY hKey;
	LONG lResult;


	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina"),
						   0,
						   KEY_QUERY_VALUE,
						   &hKey);

	if(lResult == ERROR_SUCCESS)
	{
		lResult = RegQueryValueEx(hKey,_T("hidePlugin"),
								  NULL,
								  NULL,
								  (LPBYTE) &hidePlugin,
								  &size);

		if(lResult == ERROR_SUCCESS)
		{
			if(hidePlugin == 1)
				return TRUE;
			else
				return FALSE;
		}
		else
			return FALSE;
	}
	else
		return FALSE;
}

BOOL pGinaHideMotd(void)
{
	DWORD hidePlugin = 0;
	DWORD size = 5;
	HKEY hKey;
	LONG lResult;


	lResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE,_T("SOFTWARE\\pGina"),
						   0,
						   KEY_QUERY_VALUE,
						   &hKey);

	if(lResult == ERROR_SUCCESS)
	{
		lResult = RegQueryValueEx(hKey,_T("hideMotd"),
								  NULL,
								  NULL,
								  (LPBYTE) &hidePlugin,
								  &size);

		if(lResult == ERROR_SUCCESS)
		{
			if(hidePlugin == 1)
				return TRUE;
			else
				return FALSE;
		}
		else
			return FALSE;
	}
	else
		return FALSE;
}

