/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

#include "stdafx.h"
#include "pgftp.h"
#include "profile.h"
#include "misc.h"
#include "resource.h"

// Used by CreateTempProfileDir()
typedef BOOL (WINAPI *SET_SECURITY_DESCRIPTOR_CONTROL_FUNC) (
    IN PSECURITY_DESCRIPTOR pSecurityDescriptor,
    IN SECURITY_DESCRIPTOR_CONTROL ControlBitsOfInterest,
    IN SECURITY_DESCRIPTOR_CONTROL ControlBitsToSet
);

// Used by UpdateProgress()
typedef struct {
	HWND hWndProgress;
	HWND hWndStatus;
	DWORD totalTransferred;
	UINT64 fileSize;
	TCHAR szStatus[64];
} UPDATE_PROGRESS_INFO;

void UpdateProgress(DWORD bytesTransferred, LPVOID pContext);

// See profile.h for function descriptions

int ShowProfileError(int idsReason, DWORD errDetails, BOOL logon) {
	TCHAR title[32];
	TCHAR msgFmt[256];
	TCHAR reason[256];
	TCHAR details[1024];
	TCHAR op[16];
	TCHAR tail[256];
	TCHAR msg[(sizeof(msgFmt) + sizeof(reason) + sizeof(details) +
		sizeof(op) + sizeof(tail)) / sizeof(TCHAR)];
	UINT buttons; 
	int ret;
	
	GetString(IDS_TITLE_PROFILE, title);
	GetString(IDS_MSG_PROFILE, msgFmt);
	GetString(idsReason, reason);
	FormatError(errDetails, details, sizeof(details) / sizeof(TCHAR));
	if (logon) {
		GetString(IDS_MSG_PROFILE_LOAD, op);
		GetString(IDS_MSG_PROFILE_LOAD_FOOTER, tail);
		buttons = MB_OK;
	} 
	else {
		GetString(IDS_MSG_PROFILE_SAVE, op);
		GetString(IDS_MSG_PROFILE_SAVE_FOOTER, tail);
		buttons = MB_RETRYCANCEL;
	}
	StringCbPrintf(msg, sizeof(msg), msgFmt, op, reason, details, tail);
	ret = MessageBox(NULL, msg, title,
		buttons | MB_ICONSTOP | MB_SETFOREGROUND | MB_TOPMOST);
	return ret;
}


void DeleteProfileDir(LPCTSTR profileRootDir, LPCTSTR userName) {
	SHFILEOPSTRUCT fileOp;
	TCHAR path[MAX_PATH];

	StringCchPrintf(path, MAX_PATH, TEXT("%s\\%s%c"), profileRootDir, userName, '\0');
	fileOp.hwnd = NULL;
	fileOp.wFunc = FO_DELETE;
	fileOp.pFrom = path;
	fileOp.pTo = NULL;
	fileOp.fFlags = FOF_NOCONFIRMATION | FOF_NOERRORUI | FOF_SILENT;
	fileOp.hNameMappings = NULL;
	fileOp.lpszProgressTitle = NULL;

	SHFileOperation(&fileOp);
}

BOOL LoadProfile(LPVOID hConnect,
				 PLUGIN_SETTINGS* pSettings,
				 LPCTSTR localPath, 
				 LPCTSTR remotePath, 
				 LPCTSTR defaultRemotePath,
				 HWND hWndDlg,
				 LPCTSTR userName,
		 /*OUT*/ LPTSTR archiveType)
{
	TCHAR localFile[MAX_PATH];
	TCHAR remoteFile[MAX_PATH];
	TCHAR dllPath[MAX_PATH];
	TCHAR zipCmd[MAX_PATH];
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	LPCTSTR EXTS[] = {TEXT("zip"), TEXT("7z")};
	LPCTSTR BASE_DEFAULT_NAMES[] = {TEXT("default.profile"), TEXT("profile")};
	DWORD retCode = 0;
	BOOL success = FALSE;
	BOOL fileFound = FALSE;
	UPDATE_PROGRESS_INFO upi;
	FTP_FN_TABLE* pfnTable;
	int i, j;
	
	ZeroMemory(&si, sizeof(si));
	si.cb = sizeof(si);
	ZeroMemory(&pi, sizeof(pi));
	ZeroMemory(&upi, sizeof(upi));
	pfnTable = GetFtpFnTable(pSettings->protocol);

	// Find the first match for the profile file
	for (i = 0; i < (sizeof(EXTS) / sizeof(LPCTSTR)) && !fileFound; i++) {
		StringCchPrintf(remoteFile, MAX_PATH, TEXT("%s/%s.profile.%s"), remotePath, userName, EXTS[i]);
		fileFound = pfnTable->FtpGetFileSize(hConnect, remoteFile, &upi.fileSize);
	}

	// For backwards compatibility look for "profile" .zip / .7z
	if (!fileFound) {
		for (i = 0; i < (sizeof(EXTS) / sizeof(LPCTSTR)) && !fileFound; i++) {
			StringCchPrintf(remoteFile, MAX_PATH, TEXT("%s/profile.%s"), remotePath, EXTS[i]);
			fileFound = pfnTable->FtpGetFileSize(hConnect, remoteFile, &upi.fileSize);
		}
	}

	if (!fileFound) {
		if (defaultRemotePath) {
			// Look for a default profile in the default path
			for (i = 0; i < (sizeof(EXTS) / sizeof(LPCTSTR)) && !fileFound; i++) {
				for (j = 0; j < (sizeof(BASE_DEFAULT_NAMES) / sizeof(LPCTSTR)) && !fileFound; j++) {
					StringCchPrintf(remoteFile, MAX_PATH, TEXT("%s/%s.%s"), defaultRemotePath, BASE_DEFAULT_NAMES[j], EXTS[i]);
					fileFound = pfnTable->FtpGetFileSize(hConnect, remoteFile, &upi.fileSize);
				}
			}

			if (!fileFound) {
				// Default profile is mandatory, so report failure
				ShowProfileError(IDS_ERR_PROFILE_NOT_FOUND, ERROR_FILE_NOT_FOUND, TRUE);
				goto cleanup;
			}
		}
		else {
			// Pretend we found a profile and return success. This allows
			// Windows to log the user in with a local profile, then when
			// they log out Windows will roam the profile to the temp dir
			StringCchCopy(archiveType, 4, TEXT("zip"));
			success = TRUE;
			goto cleanup;
		}
	}
	
	StringCchPrintf(localFile, MAX_PATH, TEXT("%s\\%s.profile.%s"), localPath, userName, EXTS[i-1]);
	// Remember the archive type that was selected
	StringCchCopy(archiveType, 4, EXTS[i-1]);

	upi.hWndStatus = GetDlgItem(hWndDlg, IDC_STATIC_STATUS);
	upi.hWndProgress = GetDlgItem(hWndDlg, IDC_PROGRESS1);
	// Progress bar won't work properly for files larger than 4GB.
	// If someone's roaming profile is anywhere near 4GB, they've got other problems
	// to worry about...
	SendMessage(upi.hWndProgress, PBM_SETRANGE32, 0, (DWORD)upi.fileSize); 
	ShowWindow(upi.hWndProgress, SW_SHOW);
	GetString(IDS_STATUS_DOWNLOAD, upi.szStatus);
	UpdateProgress(0, &upi);
	
	if (!pfnTable->FtpGet(hConnect, remoteFile, localFile, UpdateProgress, &upi)) 
	{
		ShowProfileError(IDS_ERR_PROFILE_DOWNLOAD, pfnTable->FtpGetLastError(hConnect), TRUE);
		goto cleanup;
	}
	
	// Unzip the profile
	ShowWindow(upi.hWndProgress, SW_HIDE);
	GetString(IDS_STATUS_UNZIP, upi.szStatus);
	SetWindowText(upi.hWndStatus, upi.szStatus);

	GetModuleFileName(g_hModule, dllPath, MAX_PATH);
	PathRemoveFileSpec(dllPath);
	StringCchPrintf(zipCmd, MAX_PATH, TEXT("%s\\7za.exe x -y \"%s\""), dllPath, localFile);
	
	if (!CreateProcess(NULL, zipCmd, NULL, NULL, TRUE, 0, NULL, localPath, &si, &pi)) {
		ShowProfileError(IDS_ERR_7ZA_INIT, GetLastError(), TRUE);
		goto cleanup;
	}
	WaitForSingleObject(pi.hProcess, INFINITE);
	GetExitCodeProcess(pi.hProcess, &retCode);
	if (retCode) {
		ShowProfileError(IDS_ERR_7ZA, ERROR_UNIDENTIFIED_ERROR, TRUE);
		goto cleanup;
	}
	
	GetString(IDS_STATUS_DONE, upi.szStatus);
	SetWindowText(upi.hWndStatus, upi.szStatus);
	success = TRUE;	

cleanup:
	if (pi.hProcess) CloseHandle(pi.hProcess);
	if (pi.hThread) CloseHandle(pi.hThread);
	DeleteFile(localFile);

	return success;
}


BOOL SaveProfile(PLUGIN_SETTINGS* pSettings,
					LPCTSTR user,
					LPCTSTR password,
					LPCTSTR localPath,
					LPCTSTR remotePath,
					HWND hWndDlg,
					LPCTSTR archiveType)
{
	LPVOID hConnect = NULL;
	BOOL success = FALSE;
	TCHAR localFile[MAX_PATH];
	TCHAR remoteFile[MAX_PATH];
	TCHAR remoteTempFile[MAX_PATH];
	TCHAR remoteBackupFile[MAX_PATH];
	TCHAR dllPath[MAX_PATH];
	TCHAR zipCmd[MAX_PATH];
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	DWORD retCode = 0;
	WIN32_FIND_DATA findData;
	HANDLE hFile;
	FTP_FN_TABLE* pfnTable;
	UPDATE_PROGRESS_INFO upi;

	ZeroMemory(&si, sizeof(si));
	si.cb = sizeof(si);
	ZeroMemory(&pi, sizeof(pi));
	ZeroMemory(&upi, sizeof(upi));
	pfnTable = GetFtpFnTable(pSettings->protocol);

	// Zip up the profile using 7za.exe
	upi.hWndStatus = GetDlgItem(hWndDlg, IDC_STATIC_STATUS);
	GetString(IDS_STATUS_ZIP, upi.szStatus);
	SetWindowText(upi.hWndStatus, upi.szStatus);
	StringCchPrintf(localFile, MAX_PATH, TEXT("%s\\%s.profile.%s"), localPath, user, archiveType);
	GetModuleFileName(g_hModule, dllPath, MAX_PATH);
	PathRemoveFileSpec(dllPath);

	StringCchPrintf(zipCmd, MAX_PATH, TEXT("%s\\7za.exe a -y -r -t%s -mx=%d \"%s\" \"*\""), dllPath, archiveType,
		pSettings->compression, localFile);
	
	if (!CreateProcess(NULL, zipCmd, NULL, NULL, TRUE, 0, NULL, localPath, &si, &pi)) {
		ShowProfileError(IDS_ERR_7ZA_INIT, GetLastError(), TRUE);
		return FALSE;
	}
	WaitForSingleObject(pi.hProcess, INFINITE);
	GetExitCodeProcess(pi.hProcess, &retCode);
	// Get information about the newly created file for later use
	hFile = FindFirstFile(localFile, &findData);
	if (retCode || !hFile) {
		ShowProfileError(IDS_ERR_7ZA, ERROR_UNIDENTIFIED_ERROR, TRUE);
		goto cleanup;
	}
	
	GetString(IDS_STATUS_CONNECTING, upi.szStatus);
	SetWindowText(upi.hWndStatus, upi.szStatus);

	// The loops give the user a chance to retry the operation in case
	// of an error before their profile changes are lost
	do {
		if (!pfnTable->FtpConnect(&hConnect, pSettings->ftpServer, pSettings->port,
				user, password,	pSettings->flags))
		{
			if (IDCANCEL == ShowProfileError(IDS_ERR_NO_CONNECTION, pfnTable->FtpGetLastError(hConnect), FALSE)) 
				goto cleanup;
		}
	} while (!hConnect);
	
	// Set up the progress bar and ftp status callback
	upi.hWndProgress = GetDlgItem(hWndDlg, IDC_PROGRESS1);
	ShowWindow(upi.hWndProgress, SW_SHOW);
	GetString(IDS_STATUS_UPLOAD, upi.szStatus);
	upi.fileSize = findData.nFileSizeLow;
	SendMessage(upi.hWndProgress, PBM_SETRANGE32, 0, findData.nFileSizeLow);
	UpdateProgress(0, &upi);

	// Create the profile path in case it doesn't exist
	pfnTable->FtpCreatePath(hConnect, remotePath); 

	// Upload profile to a temporary file
	StringCchPrintf(remoteFile, MAX_PATH, TEXT("%s/%s.profile.%s"), remotePath, user, archiveType);
	StringCchPrintf(remoteTempFile, MAX_PATH, TEXT("%s.tmp"), remoteFile);
	StringCchPrintf(remoteBackupFile, MAX_PATH, TEXT("%s.bak"), remoteFile);

	do {
		SendMessage(upi.hWndProgress, PBM_SETPOS, 0, 0);
		upi.totalTransferred = 0;
		if (!(success = pfnTable->FtpPut(hConnect, localFile, remoteTempFile, UpdateProgress, &upi))
				&& IDCANCEL == ShowProfileError(IDS_ERR_PROFILE_UPLOAD, pfnTable->FtpGetLastError(hConnect), FALSE))
		{
			goto cleanup;
		}
	} while (!success);

	GetString(IDS_STATUS_BACKUP, upi.szStatus);
	SetWindowText(upi.hWndStatus, upi.szStatus);
	// Delete any existing backup file
	pfnTable->FtpDelete(hConnect, remoteBackupFile);
	// Backup the most recent copy of the profile (if it exists)
	pfnTable->FtpRename(hConnect, remoteFile, remoteBackupFile);
	// Rename the uploaded temp file
	// TODO: Another error checking loop needs to go here, or this entire
	// block needs to go in the upload loop
	pfnTable->FtpRename(hConnect, remoteTempFile, remoteFile);
		
	GetString(IDS_STATUS_DONE, upi.szStatus);
	SetWindowText(upi.hWndStatus, upi.szStatus);
	success = TRUE;

cleanup:
	if (hConnect) pfnTable->FtpClose(hConnect);
	if (pi.hProcess) CloseHandle(pi.hProcess);
	if (pi.hThread) CloseHandle(pi.hThread);
	if (hFile) FindClose(hFile);
	return success;
}


BOOL CreateProfileDir(LPCTSTR profileRootDir, LPCTSTR userName)
{
	#define NUM_ACES 3
	INIT_CONVERSION
	LPCWSTR profileRootDirW = T2W(profileRootDir);
	SID_IDENTIFIER_AUTHORITY NtAuthority = SECURITY_NT_AUTHORITY;
	PSID administratorsSid = NULL;
	PSID localSystemSid = NULL;
	PSID userSid = NULL;
	EXPLICIT_ACCESS ace[NUM_ACES];
	PACL acl = NULL;
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	int i, ret;
	SET_SECURITY_DESCRIPTOR_CONTROL_FUNC pSetSecurityDescriptorControl;
	HMODULE hModule = NULL;
	BOOL success = FALSE;
	TCHAR path[MAX_PATH];
	DWORD userSidSize = 0;
	LPTSTR domainName = NULL;
	DWORD domainNameSize = 0;

	// Create parent directory if it doesn't exist
	ret = SHCreateDirectory(NULL, profileRootDirW);
	if (ret != ERROR_SUCCESS && ret != ERROR_ALREADY_EXISTS && ret != ERROR_FILE_EXISTS)
		return FALSE;

	// Get SID for local Administrators group
	if (!AllocateAndInitializeSid(&NtAuthority, 2, SECURITY_BUILTIN_DOMAIN_RID, 
		DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0, 0, &administratorsSid))
		return FALSE;

	// Get SID for special SYSTEM user
	if (!AllocateAndInitializeSid(&NtAuthority, 1, SECURITY_LOCAL_SYSTEM_RID,
		0, 0, 0, 0, 0, 0, 0, &localSystemSid))
		goto cleanup;
	
	// Get SID for the local Users group. Ideally, this should be the SID
	// of the user logging in, however pGina hasn't yet created the user's
	// account when this function is called. 
	if (!AllocateAndInitializeSid(&NtAuthority, 2, SECURITY_BUILTIN_DOMAIN_RID,
		DOMAIN_ALIAS_RID_USERS, 0, 0, 0, 0, 0, 0, &userSid))
		goto cleanup;

	// Create an ACL with Administrators, System and Users having 
	// full control
	ZeroMemory(&ace, NUM_ACES * sizeof(EXPLICIT_ACCESS));
	for (i = 0; i < NUM_ACES; i++) {
		ace[i].grfAccessMode = SET_ACCESS;
		ace[i].grfAccessPermissions = GENERIC_ALL;
		ace[i].grfInheritance = SUB_CONTAINERS_AND_OBJECTS_INHERIT;
		ace[i].Trustee.TrusteeForm = TRUSTEE_IS_SID;
	}

	ace[0].Trustee.ptstrName = (LPTSTR) administratorsSid;
	ace[0].Trustee.TrusteeType = TRUSTEE_IS_GROUP;
	ace[1].Trustee.ptstrName = (LPTSTR) localSystemSid;
	ace[1].Trustee.TrusteeType = TRUSTEE_IS_USER;
	ace[2].Trustee.ptstrName = (LPTSTR) userSid;
	ace[2].Trustee.TrusteeType = TRUSTEE_IS_GROUP;

	if (SetEntriesInAcl(NUM_ACES, ace, NULL, &acl))
		goto cleanup;

	// The function SetSecurityDescriptorControl is only available
	// on Win2K and above, so we need to try a runtime call
	hModule = LoadLibrary(TEXT("advapi32.dll"));
	pSetSecurityDescriptorControl = (SET_SECURITY_DESCRIPTOR_CONTROL_FUNC)
		GetProcAddress(hModule, "SetSecurityDescriptorControl");

	// Create a security descriptor for the new directory with the above ACL
	if (!InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION))
		goto cleanup;
	if (pSetSecurityDescriptorControl) {
		// This prevents automatic inheritance of ACE's from the parent 
		// object (Win2K, 2003, and XP only)
		if (!pSetSecurityDescriptorControl(&sd, SE_DACL_PROTECTED, SE_DACL_PROTECTED))
			goto cleanup;
	}
	if (!SetSecurityDescriptorDacl(&sd, TRUE, acl, FALSE))
		goto cleanup;

	// Set Administrators as owner of directory. This is necessary for
	// Windows to recognize this dir as a profile dir.
	if (!SetSecurityDescriptorOwner(&sd, administratorsSid, FALSE))
		goto cleanup;
	
	// Finally, create the folder with the user's name and above security
	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.bInheritHandle = TRUE;
	sa.lpSecurityDescriptor = &sd;
	StringCchPrintf(path, MAX_PATH, TEXT("%s\\%s"), profileRootDir, userName);
	if (!CreateDirectory(path, &sa))
		goto cleanup;

	success = TRUE;

cleanup:
	if (administratorsSid)
		FreeSid(administratorsSid);
	if (localSystemSid)
		FreeSid(localSystemSid);
	if (userSid)
		FreeSid(userSid); 
	if (domainName)
		LocalFree(domainName);
	if (hModule)
		FreeLibrary(hModule);
	if (acl)
		LocalFree(acl);

	return success;
}

void UpdateProgress(DWORD bytesTransferred, LPVOID pContext) {
	UPDATE_PROGRESS_INFO* upi = (UPDATE_PROGRESS_INFO*)pContext;
	TCHAR szStatus[64];

	upi->totalTransferred += bytesTransferred;
	SendMessage(upi->hWndProgress, PBM_SETPOS, upi->totalTransferred, 0);
	StringCbPrintf(szStatus, sizeof(szStatus), upi->szStatus, upi->totalTransferred, upi->fileSize);
	SetWindowText(upi->hWndStatus, szStatus);
}