/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

#include "stdafx.h"
#include "sftpfuncs.h"
#include "misc.h"
#include "resource.h"

#define LIBSSH2_WIN32 1

#include "libssh2/libssh2.h"
#include "libssh2/libssh2_sftp.h"

#include <sys/stat.h>

// See sftpfuncs.h for function descriptions

// Size of I/O buffer used for transferring data to / from sftp server
#define IO_BLOCK_SIZE 4096
#define IO_BLOCKS_PER_UPDATE 16

// Stores the handles that make up an SFTP connection
typedef struct {
	SOCKET hSocket;
	LIBSSH2_SESSION* hSession;
	LIBSSH2_SFTP* hSftp;
	DWORD lastError;
} SFTP_CONNECT_INFO;

typedef enum {
	ERR_TYPE_WIN32, ERR_TYPE_LIBSSH2
} ErrorTypeEnum;

void SetLastSftpError(SFTP_CONNECT_INFO* info, ErrorTypeEnum errType);

// See sftpfuncs.h for the usage of the pfnValidateHostKey parameter
BOOL pgSftpConnect(LPVOID* pphConnect, LPCTSTR server, int port, LPCTSTR user, LPCTSTR password, PFN_VALIDATE_HOST_KEY pfnValidateHostKey) {
	INIT_CONVERSION
	WSADATA wsaData;
	ADDRINFO *addrInfo = NULL, *addrInfo2;
	ADDRINFO aiHints;
	LPCSTR hostKey;
	TCHAR szHostKey[33];
	LPCSTR userA = T2A(user);
	LPCSTR passwordA = T2A(password);
	LPCSTR serverA = T2A(server);
	char portA[12];
	int i;
	SFTP_CONNECT_INFO* info;

	if (!pphConnect)
		return FALSE;

	*pphConnect = LocalAlloc(LMEM_FIXED, sizeof(SFTP_CONNECT_INFO));
	ZeroMemory(*pphConnect, sizeof(SFTP_CONNECT_INFO));
	info = *pphConnect;

	// Start winsock 2.2
	if (WSAStartup(MAKEWORD(2, 2), &wsaData)) {
		SetLastSftpError(info, ERR_TYPE_WIN32);
		return FALSE;
	}

	// Resolve the server name to a network address 
	ZeroMemory(&aiHints, sizeof(aiHints));
	aiHints.ai_family = AF_INET;
	aiHints.ai_socktype = SOCK_STREAM;
	aiHints.ai_protocol = IPPROTO_TCP;
	_itoa(port, portA, 10);
	if (getaddrinfo(serverA, portA, &aiHints, &addrInfo)) {
		SetLastSftpError(info, ERR_TYPE_WIN32);
		return FALSE;
	}
	
	// Create a socket
	if (!(info->hSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP))) {
		SetLastSftpError(info, ERR_TYPE_WIN32);
		return FALSE;
	}

	// Try each address returned in turn until we can establish a socket
	for (addrInfo2 = addrInfo ; addrInfo2 != NULL; addrInfo2 = addrInfo2->ai_next) {
		if (connect(info->hSocket, addrInfo->ai_addr, (int)addrInfo->ai_addrlen) != SOCKET_ERROR)
			break;
	}
	
	if (!addrInfo2) {  // We exhausted all the returned addresses without connecting
		SetLastSftpError(info, ERR_TYPE_WIN32);
		return FALSE;
	}

	freeaddrinfo(addrInfo);

	// Now open an SSH session on the server
	info->hSession = libssh2_session_init();
	if (libssh2_session_startup(info->hSession, (int)info->hSocket)) {
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
		return FALSE;
	}

	// Validate the host's returned key against a callback function
	if (pfnValidateHostKey) {
		// Get the host's MD5 hash and make a null-terminated, 
		// human-readable hex, TCHAR copy of it
		hostKey = libssh2_hostkey_hash(info->hSession, LIBSSH2_HOSTKEY_HASH_MD5);
		if (!hostKey) {
			SetLastSftpError(info, ERR_TYPE_LIBSSH2);
			return FALSE;
		}
		for (i = 0; i < 16; i++) {
			StringCchPrintf(szHostKey + (2 * i), 3, TEXT("%02X"), (BYTE)hostKey[i]); 
		}
		if (!pfnValidateHostKey(server, szHostKey)) {
			info->lastError = MAKE_ERROR(IDS_ERR_SSH_HOST_UNVERIFIED);
			return FALSE;
		}
	}

	// Authenticate the user
	if (libssh2_userauth_password(info->hSession, userA, passwordA)) {
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
		if (!info->lastError)
			info->lastError = ERROR_LOGON_FAILURE;
		return FALSE;
	}

	// Establish an SFTP connection
	if (!(info->hSftp = libssh2_sftp_init(info->hSession))) {
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
		return FALSE;
	}

	return TRUE;
}

DWORD pgSftpGetLastError(LPVOID hConnect) {
	if (hConnect)
		return ((SFTP_CONNECT_INFO*)hConnect)->lastError;
	else
		return ERROR_INVALID_HANDLE;
}

BOOL pgSftpGetFileSize(LPVOID hConnect, LPCTSTR filename, PUINT64 pnSize) {
	INIT_CONVERSION
	LIBSSH2_SFTP_HANDLE* hFile;
	LIBSSH2_SFTP_ATTRIBUTES attr;
	LPSTR filenameA = T2A(filename);
	SFTP_CONNECT_INFO* info = (SFTP_CONNECT_INFO*)hConnect;

	// Open the file
	if (!(hFile = libssh2_sftp_open(info->hSftp, 
			filenameA, LIBSSH2_FXF_READ, 0)))
	{
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
		return FALSE;
	}

	// Get file info
	if (libssh2_sftp_fstat(hFile, &attr)) {
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
		return FALSE;
	}

	*pnSize = attr.filesize;
	return TRUE;
}

BOOL pgSftpCreatePath(HINTERNET hConnect, LPCTSTR path) {
	INIT_CONVERSION
	LPSTR tempPath;
	LPCSTR pathA = T2A(path);
	int len = strlen(pathA);
	BOOL ret = TRUE;
	int i;
	SFTP_CONNECT_INFO* info = (SFTP_CONNECT_INFO*)hConnect;
	
	tempPath = (LPSTR)LocalAlloc(LMEM_FIXED, len + 2);
	StringCchCopyA(tempPath, len + 2, pathA);
	if (pathA[len - 1] != '/') {
		// Force path to have terminating '/'
		len++;
		tempPath[len - 1] = '/';
		tempPath[len] = '\0';
	}

	for (i = 0; i < len; i++) {
		if (tempPath[i] == '/') {
			tempPath[i] = '\0';
			ret = libssh2_sftp_mkdir(info->hSftp, tempPath, _S_IFDIR | _S_IREAD | _S_IWRITE | _S_IEXEC);
			tempPath[i] = '/';
		}
	}
	LocalFree(tempPath);

	if (ret) 
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
	return !ret;
}

BOOL pgSftpGet(LPVOID hConnect, LPCTSTR remoteFile, LPCTSTR localFile, 
			   PFN_PROGRESS pfnProgress, LPVOID pContext)
{
	INIT_CONVERSION
	LPSTR remoteFileA = T2A(remoteFile);
	BYTE buffer[IO_BLOCK_SIZE * 2];
	BYTE* readBuff = buffer;
	BYTE* writeBuff = buffer + IO_BLOCK_SIZE;
	BYTE* temp;
	HANDLE hFileLocal;
	LIBSSH2_SFTP_HANDLE* hFileRemote;
	int bytesRead = 0;
	int bytesWritten= 0;
	int progressMade = 0;
	OVERLAPPED ol;
	BOOL ret = FALSE;
	SFTP_CONNECT_INFO* info = (SFTP_CONNECT_INFO*)hConnect;

	ZeroMemory(&ol, sizeof(OVERLAPPED));

	// Open local file for writing first
	hFileLocal = CreateFile(localFile, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 
		FILE_FLAG_OVERLAPPED | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if (hFileLocal == INVALID_HANDLE_VALUE)	{
		SetLastSftpError(info, ERR_TYPE_WIN32);
		return FALSE;
	}

	// Open the remote file for reading
	if (!(hFileRemote = libssh2_sftp_open(info->hSftp, 
			remoteFileA, LIBSSH2_FXF_READ, 0))) {
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
		goto cleanup;
	}

	// This event is signaled when the asynchronous write is complete
	ol.hEvent = CreateEvent(NULL, TRUE, TRUE, NULL);

	OutputDebugString(TEXT("Testing...\n"));
	// Do the transfer
	do {
		// Read data
		if (-1 == (bytesRead = libssh2_sftp_read(hFileRemote, readBuff, IO_BLOCK_SIZE))) {
			SetLastSftpError(info, ERR_TYPE_LIBSSH2);
			goto cleanup;
		}

		// Wait for last write operation to finish
		if (!GetOverlappedResult(hFileLocal, &ol, &bytesWritten, TRUE)) {
			SetLastSftpError(info, ERR_TYPE_WIN32);
			goto cleanup;
		}
		ol.Offset += bytesWritten;
		progressMade += bytesWritten;
		if (pfnProgress && progressMade >= IO_BLOCK_SIZE * IO_BLOCKS_PER_UPDATE) {
			pfnProgress(progressMade, pContext);
			progressMade = 0;
		}

		// Swap read / write buffers
		temp = writeBuff;
		writeBuff = readBuff;
		readBuff = temp;

		// Write data asynchronously so we can continue reading immediately
		if (bytesRead) {
			WriteFile(hFileLocal, writeBuff, bytesRead, NULL, &ol);
		}
	} while (bytesRead == IO_BLOCK_SIZE);

	// Wait for final write operation to finish
	if (!GetOverlappedResult(hFileLocal, &ol, &bytesWritten, TRUE)) {
		SetLastSftpError(info, ERR_TYPE_WIN32);
		goto cleanup;
	}
	
	ret = TRUE;

cleanup:
	if (hFileLocal) CloseHandle(hFileLocal);
	if (hFileRemote) libssh2_sftp_close_handle(hFileRemote);
	if (ol.hEvent) CloseHandle(ol.hEvent);

	return ret;
}

BOOL pgSftpPut(LPVOID hConnect, LPCTSTR localFile, LPCTSTR remoteFile, 
			   PFN_PROGRESS pfnProgress, LPVOID pContext)
{
	INIT_CONVERSION
	LPSTR remoteFileA = T2A(remoteFile);
	BYTE buffer[IO_BLOCK_SIZE * 2];
	BYTE* readBuff = buffer;
	BYTE* writeBuff = buffer;
	BYTE* temp;
	HANDLE hFileLocal;
	LIBSSH2_SFTP_HANDLE* hFileRemote;
	int bytesRead = 0;
	int bytesWritten= 0;
	int progressMade = 0;
	OVERLAPPED ol;
	BOOL ret = FALSE;
	SFTP_CONNECT_INFO* info = (SFTP_CONNECT_INFO*)hConnect;

	ZeroMemory(&ol, sizeof(OVERLAPPED));

	// Open local file for reading first
	hFileLocal = CreateFile(localFile, GENERIC_READ, 0, NULL, OPEN_EXISTING, 
		FILE_FLAG_OVERLAPPED | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if (hFileLocal == INVALID_HANDLE_VALUE) {
		SetLastSftpError(info, ERR_TYPE_WIN32);
		return FALSE;
	}

	// Open the remote file for writing
	if (!(hFileRemote = libssh2_sftp_open(info->hSftp, 
			remoteFileA, LIBSSH2_FXF_WRITE | LIBSSH2_FXF_CREAT | LIBSSH2_FXF_TRUNC,
			_S_IREAD | _S_IWRITE))) 
	{
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
		goto cleanup;
	}

	// This event is signaled when the asynchronous read is complete
	ol.hEvent = CreateEvent(NULL, TRUE, TRUE, NULL);

	// Read initial chunk
	ReadFile(hFileLocal, readBuff, IO_BLOCK_SIZE, NULL, &ol);
	readBuff += IO_BLOCK_SIZE;
	
	do {
		// Wait for last read operation to finish
		if (!GetOverlappedResult(hFileLocal, &ol, &bytesRead, TRUE)) {
			SetLastSftpError(info, ERR_TYPE_WIN32);
			goto cleanup;
		}
		ol.Offset += bytesRead;

		// Read data asynchronously so we can continue writing immediately
		if (bytesRead == IO_BLOCK_SIZE) {
			ReadFile(hFileLocal, readBuff, bytesRead, NULL, &ol);
		}

		// Write data
		if (-1 == (bytesWritten = libssh2_sftp_write(hFileRemote, writeBuff, bytesRead))) {
			SetLastSftpError(info, ERR_TYPE_LIBSSH2);
			goto cleanup;
		}

		// Swap read / write buffers
		temp = writeBuff;
		writeBuff = readBuff;
		readBuff = temp;

		progressMade += bytesWritten;
		if (pfnProgress && progressMade >= IO_BLOCK_SIZE * IO_BLOCKS_PER_UPDATE) {
			pfnProgress(progressMade, pContext);
			progressMade = 0;
		}
		
	} while (bytesWritten == IO_BLOCK_SIZE);

	ret = TRUE;

cleanup:
	if (hFileLocal) CloseHandle(hFileLocal);
	if (hFileRemote) libssh2_sftp_close_handle(hFileRemote);
	if (ol.hEvent) CloseHandle(ol.hEvent);

	return ret;
}

BOOL pgSftpRename(LPVOID hConnect, LPCTSTR filename, LPCTSTR newFilename) {
	INIT_CONVERSION
	LPSTR filenameA = T2A(filename);
	LPSTR newFilenameA = T2A(newFilename);
	SFTP_CONNECT_INFO* info = (SFTP_CONNECT_INFO*)hConnect;

	if (libssh2_sftp_rename(info->hSftp, filenameA, newFilenameA)) {
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
		return FALSE;
	}

	return TRUE;
}

BOOL pgSftpDelete(LPVOID hConnect, LPCTSTR filename) {
	INIT_CONVERSION
	LPSTR filenameA = T2A(filename);
	SFTP_CONNECT_INFO* info = (SFTP_CONNECT_INFO*)hConnect;

	if (libssh2_sftp_unlink(info->hSftp, filenameA)) {
		SetLastSftpError(info, ERR_TYPE_LIBSSH2);
		return FALSE;
	}

	return TRUE;
}

void pgSftpClose(LPVOID hConnect) {
	SFTP_CONNECT_INFO * info = (SFTP_CONNECT_INFO *)hConnect;

	if (info) {
		if (info->hSftp) libssh2_sftp_shutdown(info->hSftp);
		if (info->hSession) libssh2_session_free(info->hSession);
		if (info->hSocket) closesocket(info->hSocket);
		LocalFree(hConnect);
	}
}

void SetLastSftpError(SFTP_CONNECT_INFO* info, ErrorTypeEnum errType) {
	DWORD err;

	info->lastError = 0;

	if (errType == ERR_TYPE_WIN32)
		info->lastError = GetLastError();
	else {
		// Convert libssh2 error codes to win32 codes. This makes it easier
		// to provide consistent errors between FTP and SFTP and also helps
		// with internationalization. Where there is no Win32 equivalent
		// error, a custom error is made from a string resource.
		
		err = libssh2_session_last_error(info->hSession, NULL, NULL, 0);
		if (!err)
			return;

		switch (err) {
			case LIBSSH2_ERROR_SOCKET_NONE:
				info->lastError = WSAENOTSOCK; break;
			case LIBSSH2_ERROR_BANNER_NONE:
				info->lastError = MAKE_ERROR(IDS_ERR_BANNER_NONE); break;
			case LIBSSH2_ERROR_BANNER_SEND:
				info->lastError = MAKE_ERROR(IDS_ERR_BANNER_SEND); break;
			case LIBSSH2_ERROR_KEX_FAILURE:
			case LIBSSH2_ERROR_KEY_EXCHANGE_FAILURE:
			case LIBSSH2_ERROR_HOSTKEY_INIT:
			case LIBSSH2_ERROR_HOSTKEY_SIGN:
				info->lastError = MAKE_ERROR(IDS_ERR_KEX_FAILURE); break;
			case LIBSSH2_ERROR_TIMEOUT:
			case LIBSSH2_ERROR_SOCKET_TIMEOUT:
				info->lastError = ERROR_INTERNET_TIMEOUT; break;
			case LIBSSH2_ERROR_SOCKET_DISCONNECT:
				info->lastError = ERROR_INTERNET_DISCONNECTED; break;
			case LIBSSH2_ERROR_DECRYPT:
				info->lastError = MAKE_ERROR(IDS_ERR_DECRYPT); break;
			case LIBSSH2_ERROR_PROTO:
				info->lastError = ERROR_HTTP_INVALID_SERVER_RESPONSE; break;
			case LIBSSH2_ERROR_PASSWORD_EXPIRED:
				info->lastError = ERROR_PASSWORD_EXPIRED; break;
			case LIBSSH2_ERROR_SFTP_PROTOCOL:
				err = libssh2_sftp_last_error(info->hSftp);
				switch (err) {
					case LIBSSH2_FX_NO_SUCH_FILE:
						info->lastError = ERROR_FILE_NOT_FOUND; break;
					case LIBSSH2_FX_PERMISSION_DENIED:
						info->lastError = ERROR_ACCESS_DENIED; break;
					case LIBSSH2_FX_OP_UNSUPPORTED:
						info->lastError = ERROR_NOT_SUPPORTED; break;
					case LIBSSH2_FX_INVALID_HANDLE:
						info->lastError = ERROR_INVALID_HANDLE; break;
					case LIBSSH2_FX_NO_SUCH_PATH:
						info->lastError = ERROR_PATH_NOT_FOUND; break;
					case LIBSSH2_FX_FILE_ALREADY_EXISTS:
						info->lastError = ERROR_ALREADY_EXISTS; break;
					case LIBSSH2_FX_WRITE_PROTECT:
						info->lastError = ERROR_WRITE_PROTECT; break;
					case LIBSSH2_FX_NO_MEDIA:
						info->lastError = ERROR_NO_MEDIA_IN_DRIVE; break;
					case LIBSSH2_FX_NO_SPACE_ON_FILESYSTEM:
						info->lastError = ERROR_DISK_FULL; break;
					case LIBSSH2_FX_QUOTA_EXCEEDED:
						info->lastError = ERROR_NOT_ENOUGH_QUOTA; break;
					case LIBSSH2_FX_LOCK_CONFlICT:
						info->lastError = ERROR_LOCK_VIOLATION; break;
					case LIBSSH2_FX_DIR_NOT_EMPTY:
						info->lastError = ERROR_DIR_NOT_EMPTY; break;
					case LIBSSH2_FX_NOT_A_DIRECTORY:
                        info->lastError = ERROR_DIRECTORY; break;
					case LIBSSH2_FX_INVALID_FILENAME:
						info->lastError = ERROR_INVALID_NAME; break;
					default:
						info->lastError = MAKE_ERROR(IDS_ERR_SFTP_UNKNOWN); break;
				}
				break;
			default:
				info->lastError = MAKE_ERROR(IDS_ERR_SSH_UNKNOWN); break;
		}
	}
}