/*
	This software is Copyright (C) 2004 University of Calgary

	pgFtp is free software; you can redistribute it and/or modify it under the
	terms of the GNU General Public License as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later
	version.

	This program is distributed in the hope that it will be useful, but is
	provided AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and NON-INFRINGEMENT.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	In addition, as a special exception, the copyright holders give permission
	to link the source code of this program with the OpenSSL library, and 
	distribute linked combinations including the two. You must obey the GNU 
	General Public License in all other respects for all of the code used other
	than OpenSSL.  If you modify this version of pgFtp, you may extend this 
	exception to your version, but you are not obligated to do so.  If you do not
	wish to do so, delete this exception statement from your version.

 */

#ifndef __sftpfuncs_h_
#define __sftpfuncs_h_

/*********************************************************
 * sftpfuncs.h - SFTP related functions
 *********************************************************/

// Callback function prototype used by SftpConnect for host key validation
typedef BOOL (*PFN_VALIDATE_HOST_KEY) (LPCTSTR server, LPCTSTR key);

// Callback function prototype for file transfer progress. pContext is
// the application-defined value originally passed into pgSftpGet or
// pgSftpPut.
typedef void (*PFN_PROGRESS) (DWORD bytesTransferred, LPVOID pContext);

/**
 * Opens a connection to an SFTP server. pfnValidateHostKey is a callback function
 * used to validate the host's key when an SSH connection is established. If this
 * parameter is non-null, it will call the function with a string representing the
 * MD5 hash of the host's key. The function should return TRUE if it accepts the key,
 * or FALSE otherwise. Typically, an application checks this value against a database
 * of known keys or a hard-coded value. If the key differs from the known value, a 
 * warning is usually issued to the user as this could signify a spoofing attack.
 * The returned handle should always be closed with pgSftpClose, even if the
 * connection is unsuccessful and function returns FALSE.
 **/
BOOL pgSftpConnect(LPVOID* pphConnect, LPCTSTR server, int port, LPCTSTR user, LPCTSTR password, 
				   PFN_VALIDATE_HOST_KEY pfnValidateHostKey);

/**
 * Gets a description of the last error that occured. Returns TRUE if successful,
 * FALSE otherwise.
 **/
DWORD pgSftpGetLastError(LPVOID hConnect);

/**
 * Gets the size of a remote file.
 * Returns TRUE on success, FALSE otherwise.
 **/
BOOL pgSftpGetFileSize(LPVOID hConnect, LPCTSTR filename, PUINT64 pnSize);

/**
 * Creates a directory and all parent directories on a server
 */
BOOL pgSftpCreatePath(HINTERNET hConnect, LPCTSTR path);

/**
 * Downloads a file from the sftp server. If pfnProgress is non-NULL,
 * it specifies a callback that gets periodically called as progress is made.
 * pContext is a pointer to application-defined data that gets passed to 
 * the progress function whenever it's called.
 * Returns TRUE on success, or FALSE otherwise.
 */
BOOL pgSftpGet(LPVOID hConnect, LPCTSTR remoteFile, LPCTSTR localFile, 
			   PFN_PROGRESS pfnProgress, LPVOID pContext);

/**
 * Uploads a file to the sftp server. If pfnProgress is non-NULL,
 * it specifies a callback that gets periodically called as progress is made.
 * pContext is a pointer to application-defined data that gets passed to 
 * the progress function whenever it's called.
 * Returns TRUE on success, or FALSE otherwise.
 */
BOOL pgSftpPut(LPVOID hConnect, LPCTSTR remoteFile, LPCTSTR localFile, 
			   PFN_PROGRESS pfnProgress, LPVOID pContext);

/**
 * Renames a remote file on the server.
 * Returns TRUE on success, or FALSE otherwise.
 */
BOOL pgSftpRename(LPVOID hConnect, LPCTSTR filename, LPCTSTR newFilename);

/**
 * Deletes a remote file on the server.
 * Returns TRUE on success, or FALSE otherwise.
 */
BOOL pgSftpDelete(LPVOID hConnect, LPCTSTR filename);

/**
 * Closes a connection to the remote server.
 */
void pgSftpClose(LPVOID hConnect);

#endif // __sftpfuncs_h_